module Ubc.Parse.Syntax.Import
(
parse)
where

import Control.Monad.Catch ( SomeException () )

import Data.Functor ((<&>))
import Data.List.NonEmpty (NonEmpty)

import Text.Parsec (sepBy1, ParsecT, anyChar, many, char, choice, notFollowedBy, oneOf)

import Path as Path ( Path, File, parseRelFile, Rel, parseRelDir, Dir )

import qualified Ubc.Parse.Syntax.Language as UbcLanguage
import qualified Data.List.NonEmpty as NonEmpty
import Control.Monad (liftM2)

importPath :: Monad m => ParsecT String u m (NonEmpty String)
importPath = UbcLanguage.angles (many importChar `sepBy1` char '/')
        <&> NonEmpty.fromList

importChar :: (Monad m) => ParsecT String u m Char
importChar = choice
        [ char '\\' >> oneOf ">/\\"
        , notFollowedBy (oneOf ">/") >> anyChar
        ]

parse :: ParsecT String u IO [String]
parse = do
        UbcLanguage.reserved "import"
        fragments <- importPath

        let name = Path.parseRelFile $ NonEmpty.last fragments :: Either SomeException (Path Rel File)
        let dirPath = mapM Path.parseRelDir . NonEmpty.init $ fragments :: Either SomeException [Path Rel Dir]

        case liftM2 (,) name dirPath of
                Left err -> pure []
                Right success -> pure []

